﻿const Koa = require('koa');
const cors = require('koa-cors');
const AuthBufferService = require('./AuthBuffer');
const AuthBufferConfig = require('./config');
const bodyParser = require('koa-bodyparser');

const authBufferCfg = new AuthBufferConfig();

//const appid= "1400089356";
//const auth_key = "1cfbfd2a1a03a53e";

//const authBufferT = new AuthBufferService(1400118209, "1234123", "87123456", "dk2Cz3aXKVRaK2yX");
//let userSig = authBufferT.getSignature();

const app = new Koa();
app.use(cors());
app.use(bodyParser());

// x-response-time
app.use(async (ctx, next) => {
  const start = Date.now();
  await next();
  const ms = Date.now() - start;
  ctx.set('X-Response-Time', `${ms}ms`);
});

// logger

app.use(async (ctx, next) => {
  const start = Date.now();
  await next();
  const ms = Date.now() - start;
  console.log(`${ctx.method} ${ctx.url} - ${ms}`);
});

// response

app.use(async ctx => {
  if (typeof(ctx.request.body) === 'object') {
    console.log(ctx.request.body);
    if (typeof(ctx.request.body.ask) !== 'string') {
      let reqBody = ctx.request.body;
      let sdkAppid = reqBody.sdkappid;
      let authKey = authBufferCfg.getSecKey(sdkAppid);
      const authBuffer = new AuthBufferService(sdkAppid, reqBody.roomid, reqBody.openid, authKey);
      let userSig = await authBuffer.getSignature();
      ctx.body = JSON.stringify({
        userSig: userSig,
        errorCode: 0
      });
    }
  }
});

app.listen(10005);

// 可在命令行用以下命令测试（确保系统中有curl指令）：
//
// 生成userSig:
// curl 'http://localhost:10003/' --data 'sdkappid=1400118209&roomid=1234123&openid=OPEN-T-9783813196&tinyid=9783813196'
// 返回：
// {"userSig":"eJxlj01Pg0AQhv8LZ2v2g6Vg0kPTYKi2UGxF9EKQXepIWBAWhTb*dwHRmHiYy-O8k3fmrB02*8s4SYpGqkh1pdCuNKRdjBi4kApSEFUPvZ3tzg4za25SE1NsGVNIgeyA94F-piqKfDSYUL2fCcdlCTyKVUQr-qes5lk0qmFBRwhjkyBrkqItoRJRnKrhForQN34XVQ2F7DcIwiaa-3YoyIdHMKMGMohOzJ8SOPZ4a-urtYOefY8*Zcf7mzdoVnrohExvqcyC27hJ1QMUJWOWz8LH3RKW4Z2RvLbOtW1mgWuzk3o5bAlx136d7wPv1Dmo4QLJTZB9LBba5xfoDGRh","errorCode":0}
//
// 解码userSig:
// （解码也可以访问 https://testweb.wemeet.tencent.com/webrtc/demo_for_ver2/sigCheck.html 以更方便的方式进行，但需要保证所用的SDK App ID已经联系GME添加到此网站；网站默认支持SDK App ID 1400118209）
// curl 'http://localhost:10003/' --data 'ask=decode&userSig=eJxlj8tugzAQRf*FbZpiY0NMd1AITQIERBalG4Rik1qER4yTFlX99wJFUaUuZnPOHd2ZL*XgJ4-58dhca5nJvmXKkwKUhwlzymrJC87EAPeRGy4PS3NFEIEImsYckrzuOR0C-4xommoyUEN4mBnnbctplssMCfqnrKNlNqlxAQMAIdGAOUv22XLBsryQ4y0IgF98Y6LjTT1saAASsLp3SF6Nj0AdGUBHGr6X8NOAAzd93sROh7G3T8-SOUdqWl3CdrvOGxG*ul7ywksjTN9Aj-1qF9jx5t3a2cmptIh6IWUQhapl6t6CdDFxPCdZ*NZVW*N0*0GBfSuU7x*6PGKE'
// 返回：
// {"plainText":"{\"TLS.account_type\":\"0\",\"TLS.identifier\":\"OPEN-T-9783813196\",\"TLS.tinyid\":\"9783813196\",\"TLS.roomid\":\"1234123\",\"TLS.appid_at_3rd\":\"0\",\"TLS.sdk_appid\":\"1400118209\",\"TLS.expire_after\":300,\"TLS.version\":\"20180723\",\"TLS.time\":\"1536062428\",\"TLS.sig\":\"MEQCIH0bQO3ZkgUJqiuC4XHX54x3nkVKauftWiop559Q5XYPAiAXR6cjxHFE8kVNE5zthTM22NIQsmSVOzyH0ude0nLVkw==\"}","errorCode":0}
//
// 验证userSig（返回true/false表示通过/不通过）:
//（验证同样可以访问网站，但同样需要支持的SDK App ID；默认支持1400118209）
// 返回：
// {"result":true,"errorCode":0}
